
#INCLUDE "grid.bas"

DIM SHARED AS INTEGER values(one TO seven)

TYPE tEngine
  AS INTEGER flag
  DECLARE FUNCTION Winning(g AS tGrid, p AS piece, x AS INTEGER) AS boolean
  DECLARE FUNCTION Losing(g AS tGrid, p AS piece, x AS INTEGER) AS boolean
  DECLARE FUNCTION Value(g AS tGrid, p AS piece, x AS INTEGER) AS INTEGER
  DECLARE FUNCTION CenterBonus(g AS tGrid, x AS INTEGER) AS INTEGER
  DECLARE FUNCTION Move1(g AS tGrid) AS INTEGER
  DECLARE FUNCTION Move2(g AS tGrid, p AS piece) AS INTEGER
  DECLARE FUNCTION Move3(g AS tGrid, p AS piece) AS INTEGER
  DECLARE FUNCTION Move4(g AS tGrid, p AS piece) AS INTEGER
  DECLARE FUNCTION Move5(g AS tGrid, p AS piece) AS INTEGER
END TYPE

FUNCTION tEngine.Winning(g AS tGrid, p AS piece, x AS INTEGER) AS boolean
  DIM AS boolean b(one TO seven), result = true
  DIM AS tGrid g1, g2
  DIM AS INTEGER i, j
  g1.Empty
  g2.Empty
  IF g.array(x, six) <> nothing THEN RETURN false
  FOR i = one TO seven
    b(i) = false
    g1.Copy(g.array())
    g1.Insert(p, x)
    g1.Insert(black * p, i)
    IF (g1.GridPoints(black * p) < four) THEN
      FOR j = one TO seven
        g2.Copy(g1.array())
        g2.Insert(p, j)
        IF g2.GridPoints(p) >= four THEN
          b(i) = true
          EXIT FOR
        END IF
      NEXT j
    END IF
  NEXT i
  FOR i = one TO seven
    result = result AND b(i)
  NEXT i
  FUNCTION = result
END FUNCTION

FUNCTION tEngine.Losing(g AS tGrid, p AS piece, x AS INTEGER) AS boolean
  DIM AS boolean b(one TO seven), result = false
  DIM g1 AS tGrid
  DIM AS INTEGER i
  g1.Empty
  FOR i = one TO seven
    b(i) = false
    g1.Copy(g.array())
    g1.Insert(p, x)
    b(i) = Winning(g1, black * p, i)
  NEXT i
  FOR i = one TO seven
    result = result OR b(i)
  NEXT i
  FUNCTION = result
END FUNCTION

FUNCTION tEngine.Value(g AS tGrid, p AS piece, x AS INTEGER) AS INTEGER
  DIM AS tGrid g1, g2
  DIM AS INTEGER i, j, k, l, result, a(one TO seven), b(one TO seven)
  g1.Empty
  g2.Empty
  g1.Copy(g.array())
  IF g1.Insert(p, x) THEN
    result = 0
    FOR i = one TO seven
      g1.Copy(g.array())
      g1.Insert(p, x)
      g1.Insert(black * p, i)
      a(i) = g1.GridPoints(black * p)
    NEXT i
    i = Maximum(a())
    j = Number(a(), i)
    result -= 10000 * i + 1000 * j
    FOR i = one TO seven
      g1.Copy(g.array())
      g1.Insert(p, x)
      g1.Insert(black * p, i)
      FOR j = one TO seven
        g2.Copy(g1.array())
        g2.Insert(p, j)
        b(j) = g2.GridPoints(p)
      NEXT j
      a(i) = Maximum(b())
    NEXT i
    i = Minimum(a())
    j = Number(a(), i)
    result += 100 * i + 10 * j
    result += CenterBonus(g, x)
  ELSE
    result = minInt
  END IF
  FUNCTION = result
END FUNCTION

FUNCTION tEngine.CenterBonus(g AS tGrid, x AS INTEGER) AS INTEGER
  DIM AS INTEGER y = six, result = 0
  IF g.array(x, y) = nothing THEN
    WHILE (y > one) ANDALSO (g.array(x, y - 1) = nothing)
      y -= 1
    WEND
    result = 3 - ABS(4 - x) + 2 - FIX(ABS(3.5 - y))
  END IF
  FUNCTION = result
END FUNCTION

FUNCTION tEngine.Move1(g AS tGrid) AS INTEGER
  DIM AS INTEGER result
  DO
    result = RandomInt(one, seven)
  LOOP UNTIL g.array(result, six) = nothing
  FUNCTION = result
END FUNCTION

FUNCTION tEngine.Move2(g AS tGrid, p AS piece) AS INTEGER
  DIM g1 AS tGrid
  DIM AS INTEGER i, result, a(one TO seven)
  g1.Empty
  FOR i = one TO seven
    g1.Copy(g.array())
    IF g1.Insert(p, i) ANDALSO (g1.GridPoints(p) >= four) THEN RETURN i
  NEXT i
  FOR i = one TO seven
    a(i) = Value(g, p, i)
    values(i) = a(i)
  NEXT i
  i = Maximum(a())
  result = Index(a(), i)
  FUNCTION = result
END FUNCTION

FUNCTION tEngine.Move3(g AS tGrid, p AS piece) AS INTEGER
  DIM g1 AS tGrid
  DIM AS INTEGER i, result, a(one TO seven)
  g1.Empty
  FOR i = one TO seven
    g1.Copy(g.array())
    IF g1.Insert(p, i) ANDALSO (g1.GridPoints(p) >= four) THEN RETURN i
  NEXT i
  FOR i = one TO seven
    g1.Copy(g.array())
    IF Winning(g1, p, i) THEN RETURN i
  NEXT i
  FOR i = one TO seven
    a(i) = Value(g, p, i)
    values(i) = a(i)
  NEXT i
  i = Maximum(a())
  result = Index(a(), i)
  FUNCTION = result
END FUNCTION

FUNCTION tEngine.Move4(g AS tGrid, p AS piece) AS INTEGER
  DIM g1 AS tGrid
  DIM AS INTEGER i, result, a(one TO seven)
  g1.Empty
  FOR i = one TO seven
    g1.Copy(g.array())
    IF g1.Insert(p, i) ANDALSO (g1.GridPoints(p) >= four) THEN RETURN i
  NEXT i
  FOR i = one TO seven
    g1.Copy(g.array())
    IF Winning(g1, p, i) THEN RETURN i
  NEXT i
  FOR i = one TO seven
    g1.Copy(g.array())
    IF (g1.array(i, six) = nothing) ANDALSO Losing(g1, p, i) THEN
      a(i) = minInt + 1
    ELSE
      a(i) = Value(g, p, i)
    END IF
    values(i) = a(i)
  NEXT i
  i = Maximum(a())
  result = Index(a(), i)
  FUNCTION = result
END FUNCTION

FUNCTION tEngine.Move5(g AS tGrid, p AS piece) AS INTEGER
  DIM g1 AS tGrid
  DIM AS INTEGER i, result, a(one TO seven)
  g1.Empty
  FOR i = one TO seven
    g1.Copy(g.array())
    IF g1.Insert(p, i) ANDALSO (g1.GridPoints(p) >= four) THEN RETURN i
  NEXT i
  FOR i = one TO seven
    IF (g.array(i, six) = nothing) THEN
      a(i) = CenterBonus(g, i) + RandomInt(0, 5)
    ELSE
      a(i) = -1
    END IF
    values(i) = a(i)
  NEXT i
  i = Maximum(a())
  result = Index(a(), i)
  FUNCTION = result
END FUNCTION


