
#INCLUDE "definitions.bas"
#INCLUDE "constants.bas"
#INCLUDE "functions.bas"
#INCLUDE "strings.bas"
#INCLUDE "lines.bas"

TYPE tGrid
  AS piece array(one - 1 TO seven + 1, one - 1 TO six + 1)
  DECLARE SUB Empty
  DECLARE SUB Copy(p() AS piece)
  DECLARE FUNCTION IsFull AS boolean
  DECLARE FUNCTION IsEmpty AS boolean
  DECLARE FUNCTION Insert(p AS piece, x AS INTEGER) AS boolean
  DECLARE FUNCTION Fill(s AS STRING) AS boolean
  DECLARE FUNCTION LinePoints(l AS _line, p AS piece) AS INTEGER
  DECLARE FUNCTION GridPoints(p AS piece) AS INTEGER
  DECLARE FUNCTION AsString AS STRING
END TYPE

SUB tGrid.Empty
  DIM AS INTEGER x, y
  FOR x = one - 1 TO seven + 1
    FOR y = one - 1 TO six + 1
      array(x, y) = border
    NEXT y
  NEXT x
  FOR x = one TO seven
    FOR y = one TO six
      array(x, y) = nothing
    NEXT y
  NEXT x
END SUB

SUB tGrid.Copy(p() AS piece)
  FOR x AS INTEGER = one TO seven
    FOR y AS INTEGER = one TO six
      array(x, y) = p(x, y)
    NEXT y
  NEXT x
END SUB

FUNCTION tGrid.IsFull AS boolean
  DIM result AS boolean = true
  DIM x AS INTEGER = one
  DO WHILE (x <= seven) AND result
    IF array(x, six) = nothing THEN result = false
    x += 1
  LOOP
  RETURN result
END FUNCTION

FUNCTION tGrid.IsEmpty AS boolean
  DIM result AS boolean = true
  DIM x AS INTEGER = one
  DO WHILE (x <= seven) AND result
    IF array(x, six) <> nothing THEN result = false
    x += 1
  LOOP
  RETURN result
END FUNCTION

FUNCTION tGrid.Insert(p AS piece, x AS INTEGER) AS boolean
  DIM y AS INTEGER = six
  DIM result AS boolean = (array(x, y) = nothing)
  IF result THEN
    DO WHILE (array(x, y - 1) = nothing)
      y -= 1
    LOOP
    array(x, y) = p
  END IF
  RETURN result
END FUNCTION

FUNCTION tGrid.Fill(s AS STRING) AS boolean
  DIM result AS boolean = IsEmpty AND IsValidFormat(s) ANDALSO IsValidSequence(s)
  DIM p AS piece = white
  IF result THEN
    FOR i AS INTEGER = 0 TO LEN(s) - 1
      Insert(p, AscToInt(s[i]))
      p *= black
    NEXT i
  END IF
  RETURN result
END FUNCTION

FUNCTION tGrid.LinePoints(l AS _line, p AS piece) AS INTEGER
  DIM AS INTEGER x, y, n=0, o=0, nmx=0, omx=0, result
  x = l.x1
  y = l.y1
  DO WHILE array(x, y) <> border
    IF array(x, y) = p THEN n += 1 ELSE n = 0
    IF array(x, y) <> black * p THEN o += 1 ELSE o = 0
    IF n > nmx THEN nmx = n
    IF o > omx THEN omx = o
    x += l.dx
    y += l.dy 
  LOOP
  IF omx < 4 THEN result = 0 ELSE result = nmx
  RETURN result
END FUNCTION

FUNCTION tGrid.GridPoints(p AS piece) AS INTEGER
  DIM AS INTEGER i, j, k = 0
  FOR i = 0 TO 24
    j = LinePoints(lines(i), p)
	IF j > k THEN k = j
  NEXT i
  RETURN k
END FUNCTION

FUNCTION tGrid.AsString AS STRING
  DIM result AS STRING = ""
  FOR y AS INTEGER = six + 1 TO one - 1 STEP -1
    FOR x AS INTEGER = one - 1 TO seven + 1
      SELECT CASE array(x, y)
        CASE nothing
          result = result & " "
        CASE white
          result = result & "X"
        CASE black
          result = result & "O"
        CASE border
          result = result & "#"
      END SELECT
    NEXT x
    result = result & CHR(13, 10)
  NEXT y
  RETURN result
END FUNCTION


